import unittest
from unittest.mock import patch


class TestStats(unittest.TestCase):

    def test_import_error(self):
        patched = {'aiostatsd.client': None,
                   'aiostatsd': None}

        with patch.dict('sys.modules', patched):
            from molotov.stats import get_statsd_client
            self.assertRaises(ImportError, get_statsd_client)



import json
import socket
import os
import sys
from contextlib import contextmanager
from urllib.parse import urlparse, urlunparse
from socket import gethostbyname


@contextmanager
def stream_log(msg, pid=True):
    if pid:
        msg = '[%d] %s...' % (os.getpid(), msg)
    sys.stdout.write(msg)
    sys.stdout.flush()

    yield

    sys.stdout.write('OK\n')
    sys.stdout.flush()


def log(msg, pid=True):
    if pid:
        print('[%d] %s' % (os.getpid(), msg))
    else:
        print(msg)


_DNS_CACHE = {}


def resolve(url):
    parts = urlparse(url)

    if '@' in parts.netloc:
        username, password = parts.username, parts.password
        netloc = parts.netloc.split('@', 1)[1]
    else:
        username, password = None, None
        netloc = parts.netloc

    if ':' in netloc:
        host = netloc.split(':')[0]
    else:
        host = netloc

    port_provided = False
    if not parts.port and parts.scheme == 'https':
        port = 443
    elif not parts.port and parts.scheme == 'http':
        port = 80
    else:
        port = parts.port
        port_provided = True

    original = host
    resolved = None
    if host in _DNS_CACHE:
        resolved = _DNS_CACHE[host]
    else:
        try:
            resolved = gethostbyname(host)
            _DNS_CACHE[host] = resolved
        except socket.gaierror:
            return url, original, host

    # Don't use a resolved hostname for SSL requests otherwise the
    # certificate will not match the IP address (resolved)
    host = resolved if parts.scheme != 'https' else host
    netloc = host
    if port_provided:
        netloc += ':%d' % port
    if username is not None:
        if password is not None:
            netloc = '%s:%s@%s' % (username, password, netloc)
        else:
            netloc = '%s@%s' % (username, netloc)

    if port not in (443, 80):
        host += ':%d' % port
        original += ':%d' % port

    new = urlunparse((parts.scheme, netloc, parts.path or '', '',
                      parts.query or '', parts.fragment or ''))
    return new, original, host


class OptionError(Exception):
    pass


def _expand_args(args, options):
    for key, val in options.items():
        setattr(args, key, val)


def expand_options(config, scenario, args):
    if not isinstance(config, str):
        try:
            config = json.loads(config.read())
        except ValueError:
            raise OptionError("Can't parse %r" % config)
    else:
        if not os.path.exists(config):
            raise OptionError("Can't find %r" % config)

        with open(config) as f:
            try:
                config = json.loads(f.read())
            except ValueError:
                raise OptionError("Can't parse %r" % config)

    if 'molotov' not in config:
        raise OptionError("Bad config -- no molotov key")

    if 'tests' not in config['molotov']:
        raise OptionError("Bad config -- no molotov/tests key")

    if scenario not in config['molotov']['tests']:
        raise OptionError("Can't find %r in the config" % scenario)

    _expand_args(args, config['molotov']['tests'][scenario])



import os
import asyncio
from collections import namedtuple

from molotov.api import scenario
from molotov.tests.support import TestLoop, coserver, dedicatedloop, set_args
from molotov.tests.statsd import UDPServer
from molotov.run import run, main
from molotov import __version__


_CONFIG = os.path.join(os.path.dirname(__file__), 'molotov.json')
_RES = []


class TestRunner(TestLoop):
    def setUp(self):
        super(TestRunner, self).setUp()
        _RES[:] = []

    @dedicatedloop
    def test_runner(self):
        test_loop = asyncio.get_event_loop()
        test_loop.set_debug(True)
        test_loop._close = test_loop.close
        test_loop.close = lambda: None

        @scenario(10)
        async def here_one(session):
            async with session.get('http://localhost:8888') as resp:
                await resp.text()
            _RES.append(1)

        @scenario(90)
        async def here_two(session):
            if session.statsd is not None:
                session.statsd.incr('yopla')
            _RES.append(2)

        args = namedtuple('args', 'verbose quiet duration exception')
        args.verbose = 1
        args.quiet = False
        args.duration = 1
        args.exception = True
        args.console = True
        args.processes = 1
        args.workers = 1
        args.debug = True
        args.statsd = True
        args.statsd_server = '127.0.0.1'
        args.statsd_port = 9999
        args.scenario = 'molotov.tests.test_run'
        args.single_mode = None
        args.max_runs = None

        server = UDPServer('127.0.0.1', 9999, loop=test_loop)
        _stop = asyncio.Future()

        async def stop():
            await _stop
            await server.stop()

        server_task = asyncio.ensure_future(server.run())
        stop_task = asyncio.ensure_future(stop())

        with coserver():
            run(args)

        _stop.set_result(True)
        test_loop.run_until_complete(asyncio.gather(server_task, stop_task))

        self.assertTrue(len(_RES) > 0)

        udp = server.flush()
        self.assertTrue(len(udp) > 0)
        test_loop._close()

    @dedicatedloop
    def test_main(self):
        with set_args('molotov', '-cq', '-d', '1', 'molotov/tests/example.py'):
            main()

    def _test_molotov(self, *args):
        with set_args('molotov', *args) as (stdout, stderr):
            try:
                main()
            except SystemExit:
                pass
        return stdout.read().strip(), stderr.read().strip()

    @dedicatedloop
    def test_version(self):
        stdout, stderr = self._test_molotov('--version')
        self.assertEqual(stdout, __version__)

    @dedicatedloop
    def test_empty_scenario(self):
        stdout, stderr = self._test_molotov('')
        self.assertTrue('Cannot import' in stdout)

    @dedicatedloop
    def test_no_scenario(self):
        stdout, stderr = self._test_molotov()
        self.assertTrue('Cannot import' in stdout)

    @dedicatedloop
    def test_config_no_scenario(self):
        stdout, stderr = self._test_molotov('-c', '--config', _CONFIG,
                                            'DONTEXIST')
        wanted = "Can't find 'DONTEXIST' in the config"
        self.assertTrue(wanted in stdout)

    @dedicatedloop
    def test_config_verbose(self):
        stdout, stderr = self._test_molotov('-v', '--config', _CONFIG)
        wanted = "You have to be in console mode"
        self.assertTrue(wanted in stdout)

    @dedicatedloop
    def test_config_verbose_quiet(self):
        stdout, stderr = self._test_molotov('-qv', '--config', _CONFIG)
        wanted = "You can't"
        self.assertTrue(wanted in stdout)

    @dedicatedloop
    def test_config_no_scenario_found(self):
        stdout, stderr = self._test_molotov('-c', 'molotov.tests.test_run')
        wanted = "No scenario was found"
        self.assertTrue(wanted in stdout)

    @dedicatedloop
    def test_config_no_single_mode_found(self):

        @scenario(10)
        async def not_me(session):
            _RES.append(3)

        stdout, stderr = self._test_molotov('-c', '-s', 'blah',
                                            'molotov.tests.test_run')
        wanted = "Can't find"
        self.assertTrue(wanted in stdout)

    @dedicatedloop
    def test_single_mode(self):

        @scenario(10)
        async def here_three(session):
            _RES.append(3)

        stdout, stderr = self._test_molotov('-cx', '--max-runs', '2', '-s',
                                            'here_three',
                                            'molotov.tests.test_run')
        wanted = "SUCCESSES: 2"
        self.assertTrue(wanted in stdout)



try:
    from molotov.api import (scenario, setup, global_setup, teardown,  # NOQA
                             global_teardown)                          # NOQA
except ImportError:
    pass   # first import

__version__ = '0.6.2'



import os
import sys
import argparse

from importlib import import_module
from importlib.util import spec_from_file_location, module_from_spec

from molotov.fmwk import runner
from molotov.api import get_scenarios, get_scenario
from molotov import __version__
from molotov.util import log, expand_options, OptionError
from molotov import ui


def _parser():
    parser = argparse.ArgumentParser(description='Load test.')

    parser.add_argument('scenario', default="loadtest.py",
                        help="path or module name that contains scenarii",
                        nargs="?")

    parser.add_argument('-s', '--single-mode', default=None, type=str,
                        help="Name of a single scenario to run once.")

    parser.add_argument('--config', default=None, type=str,
                        help='Point to a JSON config file.')

    parser.add_argument('--version', action='store_true', default=False,
                        help='Displays version and exits.')

    parser.add_argument('--debug', action='store_true', default=False,
                        help='Run the event loop in debug mode.')

    parser.add_argument('-v', '--verbose', action='count', default=0,
                        help=('Verbosity level. -v will display '
                              'tracebacks. -vv requests and responses.'))

    parser.add_argument('-w', '--workers', help='Number of workers',
                        type=int, default=1)

    parser.add_argument('-p', '--processes', help='Number of processes',
                        type=int, default=1)

    parser.add_argument('-d', '--duration', help='Duration in seconds',
                        type=int, default=86400)

    parser.add_argument('-r', '--max-runs', help='Maximum runs per worker',
                        type=int, default=None)

    parser.add_argument('-q', '--quiet', action='store_true', default=False,
                        help='Quiet')

    parser.add_argument('-x', '--exception', action='store_true',
                        default=False,
                        help='Stop on first failure.')

    parser.add_argument('-c', '--console', action='store_true',
                        default=False,
                        help='Use simple console for feedback')

    parser.add_argument('--statsd', help='Activates statsd',
                        action='store_true', default=False)

    parser.add_argument('--statsd-server', help='Statsd Server',
                        type=str, default="127.0.0.1")

    parser.add_argument('--statsd-port', help='Statsd Port',
                        type=int, default=8125)

    return parser


def main():
    parser = _parser()
    args = parser.parse_args()

    if args.version:
        print(__version__)
        sys.exit(0)

    if args.config:
        if args.scenario == 'loadtest.py':
            args.scenario = 'test'

        try:
            expand_options(args.config, args.scenario, args)
        except OptionError as e:
            print(str(e))
            sys.exit(0)

    if args.statsd:
        # early import to quit if no aiostatsd
        from aiostatsd.client import StatsdClient
        if StatsdClient is None:
            print('You need to install aiostatsd when using --statds')
            sys.exit(0)

    return run(args)


def run(args):
    if not args.quiet:
        log('**** Molotov v%s. Happy breaking! ****' % __version__, pid=None)

    if os.path.exists(args.scenario):
        spec = spec_from_file_location("loadtest", args.scenario)
        module = module_from_spec(spec)
        spec.loader.exec_module(module)
    else:
        try:
            import_module(args.scenario)
        except (ImportError, ValueError):
            print('Cannot import %r' % args.scenario)
            sys.exit(1)

    if len(get_scenarios()) == 0:
        print('You need at least one scenario. No scenario was found.')
        print('A scenario with a weight of 0 is ignored')
        sys.exit(1)

    if args.verbose > 0 and args.quiet:
        print("You can't use -q and -v at the same time")
        sys.exit(1)

    if args.verbose > 0 and not args.console:
        print("You have to be in console mode (-c) to use -v")
        sys.exit(1)

    if args.single_mode:
        if get_scenario(args.single_mode) is None:
            print("Can't find %r in registered scenarii" % args.single_mode)
            sys.exit(1)

    res = runner(args, screen=ui.init_screen)

    if not args.quiet:
        print('SUCCESSES: %(OK)d | FAILURES: %(FAILED)d\r' % res)
        print('*** Bye ***')



from molotov import __version__
from molotov.slave import main
from molotov.tests.support import TestLoop, dedicatedloop, set_args


_REPO = 'https://github.com/loads/molotov'


class TestSlave(TestLoop):
    @dedicatedloop
    def test_main(self):
        with set_args('moloslave', _REPO, 'test') as out:
            main()
        output = out[0].read()
        self.assertTrue('Preparing 1 workers...OK' in output)

    @dedicatedloop
    def test_fail(self):
        with set_args('moloslave', _REPO, 'fail'):
            self.assertRaises(Exception, main)

    @dedicatedloop
    def test_version(self):
        with set_args('moloslave', '--version') as out:
            try:
                main()
            except SystemExit:
                pass
        version = out[0].read().strip()
        self.assertTrue(version, __version__)



from molotov import scenario


_API = 'http://localhost:8080'


@scenario(40)
async def scenario_one(session):
    async with session.get(_API) as resp:
        res = await resp.json()
        assert res['result'] == 'OK'
        assert resp.status == 200


@scenario(60)
async def scenario_two(session):
    async with session.get(_API) as resp:
        assert resp.status == 200





def get_statsd_client(server="127.0.0.1", port=8125, packet_size=512,
                      flush_interval=.01):
    from aiostatsd.client import StatsdClient
    client = StatsdClient(server, port, packet_size=packet_size,
                          flush_interval=flush_interval)
    return client



from io import StringIO
import json
import unittest
import os
from molotov.util import resolve, expand_options, OptionError

_HERE = os.path.dirname(__file__)
config = os.path.join(_HERE, '..', '..', 'molotov.json')


class Args:
    pass


class TestUtil(unittest.TestCase):
    def test_resolve(self):

        urls = [('http://localhost:80/blah', 'http://127.0.0.1:80/blah'),
                ('https://localhost', 'https://localhost'),
                ('http://cantfind', 'http://cantfind'),
                ('https://google.com', 'https://google.com'),
                ('http://user:pass@localhost/blah?yeah=1#ok',
                 'http://user:pass@127.0.0.1/blah?yeah=1#ok'),
                ('http://tarek@localhost/blah',
                 'http://tarek@127.0.0.1/blah')]

        for url, wanted in urls:
            changed, original, resolved = resolve(url)
            self.assertEqual(changed, wanted,
                             '%s vs %s' % (original, resolved))

    def test_config(self):
        args = Args()
        expand_options(config, "test", args)
        self.assertEqual(args.duration, 1)

    def _get_config(self, data):
        data = json.dumps(data)
        data = StringIO(data)
        data.seek(0)
        return data

    def test_bad_config(self):
        args = Args()

        bad_data = [({}, 'test'),
                    ({'molotov': {}}, 'test'),
                    ({'molotov': {'tests': {}}}, 'test')]

        for data, scenario in bad_data:
            self.assertRaises(OptionError, expand_options,
                              self._get_config(data), scenario, args)



import unittest
import time
import os

from molotov.ui import init_screen, quit
from molotov.result import LiveResults


class TestUI(unittest.TestCase):

    def test_init_screen(self):
        results = LiveResults()

        called = []
        pids = [1, 2, 3, os.getpid()]
        running = time.time()

        def get_results(*args):
            called.append(True)
            # give the updater a chance to kick in
            if time.time() - running > 2:
                quit()
            else:
                results.incr_success()
                results.incr_failure()

            return results

        ui = init_screen(pids, get_results)
        ui.run()
        self.assertTrue(len(called) > 1)



import asyncio


# taken from aiostatsd.tests.test_client
class ServerProto:
    def __init__(self, received_queue):
        self.received_queue = received_queue
        self.transport = None

    def connection_made(self, transport):
        self.transport = transport

    def datagram_received(self, data, addr):
        self.received_queue.put_nowait(data)

    def disconnect(self):
        self.transport.close()

    def error_received(self, exc):
        raise Exception(exc)

    def connection_lost(self, exc):
        print(exc)


class UDPServer(object):
    def __init__(self, host, port, loop=None):
        self.host = host
        self.port = port
        if loop is None:
            self.loop = asyncio.get_event_loop()
        else:
            self.loop = loop
        self._stop = asyncio.Future(loop=self.loop)
        self._done = asyncio.Future(loop=self.loop)
        self.incoming = asyncio.Queue(loop=self.loop)

    async def run(self):
        ctx = {}

        def make_proto():
            proto = ServerProto(self.incoming)
            ctx['proto'] = proto
            return proto

        conn = self.loop.create_datagram_endpoint(
            make_proto, local_addr=(self.host, self.port)
        )

        async def listen_for_stop():
            await self._stop
            ctx['proto'].disconnect()

        await asyncio.gather(conn, listen_for_stop(), loop=self.loop)
        self._done.set_result(True)

    def flush(self):
        out = []
        while not self.incoming.empty():
            out.append(self.incoming.get_nowait())
        return out

    async def stop(self):
        self._stop.set_result(True)
        await self._done



import json
import os
import sys
import argparse
import subprocess
import tempfile
import shutil
import site

from molotov import __version__
from molotov.run import run, _parser


def clone_repo(github):
    # XXX security
    subprocess.check_call('git clone %s .' % github, shell=True)


def create_virtualenv(virtualenv, python):
    # XXX security
    subprocess.check_call('%s --python %s venv' % (virtualenv, python),
                          shell=True)


def install_reqs(reqfile):
    subprocess.check_call('./venv/bin/pip install -r %s' % reqfile,
                          shell=True)


def run_test(**options):
    parser = _parser()
    fields = {}
    cli = []

    for action in parser._actions:
        if action.dest in ('help', 'scenario'):
            continue
        op_str = action.option_strings[0]
        fields[action.dest] = op_str, action.const

    for key, value in options.items():
        if key in fields:
            opt, const = fields[key]
            if const:
                cli.append(opt)
            else:
                cli.append(opt)
                cli.append(str(value))

    cli.append(options.pop('scenario', 'loadtest.py'))
    args = parser.parse_args(args=cli)
    print('Running: molotov %s' % ' '.join(cli))
    return run(args)


def main():
    parser = argparse.ArgumentParser(description='Github-based load test')

    parser.add_argument('--version', action='store_true', default=False,
                        help='Displays version and exits.')

    parser.add_argument('--virtualenv', type=str, default='virtualenv',
                        help='Virtualenv executable.')

    parser.add_argument('--python', type=str, default=sys.executable,
                        help='Python executable.')

    parser.add_argument('--config', type=str, default='molotov.json',
                        help='Path of the configuration file.')

    parser.add_argument('repo', help='Github repo', type=str, nargs="?")
    parser.add_argument('run', help='Test to run', nargs="?")

    args = parser.parse_args()

    if args.version:
        print(__version__)
        sys.exit(0)

    tempdir = tempfile.mkdtemp()
    curdir = os.getcwd()
    os.chdir(tempdir)
    print('Working directory is %s' % tempdir)
    try:
        clone_repo(args.repo)
        config_file = os.path.join(tempdir, args.config)

        with open(config_file) as f:
            config = json.loads(f.read())

        # creating the virtualenv
        create_virtualenv(args.virtualenv, args.python)

        # install deps
        if 'requirements' in config['molotov']:
            install_reqs(config['molotov']['requirements'])

        # load deps into sys.path
        pyver = '%d.%d' % (sys.version_info.major, sys.version_info.minor)
        site.addsitedir(os.path.join(tempdir,
                        'venv', 'lib', 'python' + pyver, 'site-packages'))

        # environment
        if 'env' in config['molotov']:
            for key, value in config['molotov']['env'].items():
                os.environ[key] = value

        run_test(**config['molotov']['tests'][args.run])
    except Exception:
        os.chdir(curdir)
        shutil.rmtree(tempdir, ignore_errors=True)
        raise



from functools import partial
from humanize import naturaltime
from molotov import __version__
from molotov.result import ClosedError

import urwid


_REFRESH = .3


def unhandled(key):
    if key == 'ctrl c':
        raise urwid.ExitMainLoop


def quit(*args):
    raise urwid.ExitMainLoop


palette = [
    ('header', 'white', 'dark gray'),
    ('body', 'black', 'white'),
    ('footer', 'white', 'dark gray')]


def process_box(procid, refresh=None, loop=None):

    def update_box(body, footer, refresh, loop, *args):
        try:
            results = refresh()
            results.update()
            oks = results.get_successes(procid)
            fails = results.get_failures(procid)
        except ClosedError:
            loop.set_alarm_in(.1, quit)
            return

        body.set_text('SUCCESSES: %d | FAILURES: %d' % (oks, fails))
        duration = 'Started %s.' % naturaltime(results.howlong())
        footer.base_widget.set_text(duration)
        updater = partial(update_box, body, footer, refresh)
        loop.set_alarm_in(_REFRESH, updater)

    header = urwid.Padding(urwid.Text('Process [%d]' % procid), left=1)
    header = urwid.AttrWrap(header, 'header')
    body = urwid.AttrWrap(urwid.Text('', align='center'), 'body')
    footer = urwid.Padding(urwid.Text(''), left=1)
    footer = urwid.AttrWrap(footer, 'footer')
    divider = urwid.Divider()
    frame = urwid.Pile([('pack', header), divider, body, divider,
                        ('pack', footer)])

    updater = partial(update_box, body, footer, refresh)
    return frame, updater


def init_screen(procs, updater, loop=None):
    widgets = []
    updaters = []

    for proc in procs:
        partial(updater, proc)
        widget, updating = process_box(proc, updater, loop)
        widgets.append(widget)
        updaters.append(updating)

    main_header = urwid.Text('Molotov v%s - Happy Breaking !' % __version__)
    main_footer = urwid.Text('Use Ctrl+C to quit')
    divider = urwid.Divider()

    main_widget = urwid.GridFlow(cells=widgets, cell_width=35, h_sep=1,
                                 v_sep=1, align='left')

    screen = urwid.Pile([main_header, divider, main_widget, divider,
                         main_footer])
    screen = urwid.Filler(screen, 'top')

    if loop is not None:
        loop = urwid.AsyncioEventLoop(loop=loop)

    urwid_loop = urwid.MainLoop(
            screen,
            palette,
            event_loop=loop,
            unhandled_input=unhandled,
    )
    for updating in updaters:
        urwid_loop.set_alarm_in(1, updating)
    return urwid_loop



import os
import time
from queue import Empty
from multiprocessing import Queue


def _now():
    return int(time.time())


class ClosedError(Exception):
    pass


class LiveResults:
    def __init__(self, use_buffer=True, buffer_max_age=1.):
        self._data = {os.getpid(): {'OK': 0, 'FAILED': 0}}
        self.start = _now()
        self.queue = Queue()
        self.closed = False
        self._buffer = {'OK': 0, 'FAILED': 0}
        self._buffer_age = time.time()
        self._bma = buffer_max_age
        self._use_buffer = use_buffer

    def close(self):
        self.closed = True

    def update(self):
        if self.closed:
            raise ClosedError()
        try:
            if not self.queue.empty():
                while True:
                    try:
                        data = self.queue.get_nowait()
                    except Empty:
                        break
                    if data is not None:
                        pid, res, count = data
                        if pid not in self._data:
                            self._data[pid] = {'OK': 0, 'FAILED': 0}
                        if res == 'OK':
                            self._data[pid]['OK'] += count
                        if res == 'FAILED':
                            self._data[pid]['FAILED'] += count
        except BrokenPipeError:
            pass

    def get_successes(self, pid=None):
        return self._count('OK', pid)

    def get_failures(self, pid=None):
        return self._count('FAILED', pid)

    def _count(self, value, pid=None):
        if pid is not None:
            res = self._data.get(pid)
            if res is not None:
                return res[value]
            return 0
        count = 0
        for pid, values in self._data.items():
            count += values[value]
        return count

    # thread-safeness?
    def _unbuffer(self):
        if time.time() - self._buffer_age < self._bma:
            return
        pid = os.getpid()
        ok_count = self._buffer['OK']
        self._buffer['OK'] = 0
        failed_count = self._buffer['FAILED']
        self._buffer['FAILED'] = 0
        self._buffer_age = time.time()
        try:
            self.queue.put((pid, 'OK', ok_count))
            self.queue.put((pid, 'FAILED', failed_count))
        except BrokenPipeError:
            pass

    def _incr(self, counter, count=1):
        if self.closed:
            raise ClosedError()
        if self._use_buffer:
            self._buffer[counter] += count
            self._unbuffer()
        else:
            self.queue.put((os.getpid(), counter, count))

    def incr_success(self, count=1):
        self._incr('OK', count)

    def incr_failure(self, count=1):
        self._incr('FAILED', count)

    def howlong(self):
        return _now() - self.start

    def __str__(self):
        self.update()
        return 'SUCCESSES: %s | FAILURES: %s' % (self.get_successes(),
                                                 self.get_failures())



import tempfile
import unittest
import shutil
import os

from molotov import quickstart, __version__
from molotov.tests.support import set_args


class TestQuickStart(unittest.TestCase):
    def setUp(self):
        self.tempdir = tempfile.mkdtemp()
        self._answers = ['y', 'welp', self.tempdir]

    def tearDown(self):
        shutil.rmtree(self.tempdir)

    def _input(self, text):
        if self._answers == []:
            self._answers = ['y', 'welp', self.tempdir]
        answer = self._answers.pop()
        return answer

    def test_version(self):
        quickstart._input = self._input

        with set_args('molostart', '--version') as out:
            try:
                quickstart.main()
            except SystemExit:
                pass
        output = out[0].read().strip()
        self.assertEqual(output, __version__)

    def test_generate(self):
        quickstart._input = self._input

        with set_args('molostart'):
            quickstart.main()

        result = os.listdir(self.tempdir)
        result.sort()
        self.assertEqual(result, ['Makefile', 'loadtest.py', 'molotov.json'])

        # second runs stops
        with set_args('molostart'):
            try:
                quickstart.main()
                raise AssertionError()
            except SystemExit:
                pass
